package com.winhc.data.push.controller;

import cn.hutool.core.lang.Assert;
import com.aliyun.openservices.shade.org.apache.commons.lang3.StringUtils;
import com.winhc.data.push.common.TaskEnum;
import com.winhc.data.push.constant.BaseParam;
import com.winhc.data.push.service.SynDataService;
import com.winhc.data.push.utils.BaseUtils;
import com.winhc.data.push.vo.ResponseVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import static com.winhc.data.push.constant.BaseParam.SYN_ANSHUO_UPLOAD_TASKS;


/**
 * @author π
 * @Description:推送数据
 * @date 2022/7/20 16:24
 */
@Slf4j
@AllArgsConstructor
@RestController
@Api(tags = "任务触发", value = "task")
@RequestMapping("task")
public class PushDataController {

    private final SynDataService synDataService;


    @ApiOperation(value = "触发任务")
    @GetMapping("push/{ds}")
    public ResponseVo touch(@PathVariable String ds, @RequestParam String tn, @RequestParam String sign) {
        //日期是昨天之前数据
        Assert.isTrue(BaseUtils.verifyDate(ds), "ds 不合法");
        Assert.isTrue(BaseParam.sign.equals(sign), "验签不通过！");
        Assert.isTrue(StringUtils.isNotBlank(tn), "参数不能为空!");
        long start = System.currentTimeMillis();
        try {
            String re = synDataService.saveTask(tn, ds);
            return ResponseVo.success(start, re);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return ResponseVo.failure(start, e.getMessage());
        }
    }

    @ApiOperation(value = "强制触发任务")
    @GetMapping("repush/{ds}")
    public ResponseVo reTouch(@PathVariable String ds, @RequestParam String tn, @RequestParam String sign) {
        Assert.isTrue(BaseUtils.verifyDate(ds), "ds 不合法");
        Assert.isTrue(BaseParam.sign.equals(sign), "验签不通过！");
        Assert.isTrue(StringUtils.isNotBlank(tn), "参数不能为空!");
        long start = System.currentTimeMillis();
        try {
            Boolean re = synDataService.updateTask(tn, ds, "create");
            return ResponseVo.success(start, re ? "restart" : "error");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return ResponseVo.failure(start, e.getMessage());
        }
    }


    @ApiOperation(value = "上传数据")
    @GetMapping("upload/{ds}")
    public ResponseVo upload(@PathVariable String ds, @RequestParam String sign) {
        //日期是昨天之前数据
        Assert.isTrue(BaseUtils.verifyDate(ds), "ds 不合法");
        Assert.isTrue(BaseParam.sign.equals(sign), "验签不通过！");
        long start = System.currentTimeMillis();
        try {
            String re = synDataService.uploadTask(ds);
            return ResponseVo.success(start, re);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return ResponseVo.failure(start, e.getMessage());
        }
    }

    @ApiOperation(value = "强制上传数据")
    @GetMapping("reupload/{ds}")
    public ResponseVo reupload(@PathVariable String ds, @RequestParam String sign) {
        Assert.isTrue(BaseUtils.verifyDate(ds), "ds 不合法");
        Assert.isTrue(BaseParam.sign.equals(sign), "验签不通过！");
        long start = System.currentTimeMillis();
        try {
            Boolean re = synDataService.updateTaskStatus(SYN_ANSHUO_UPLOAD_TASKS, ds, TaskEnum.STATUS.CREATE.code);
            return ResponseVo.success(start, re ? "restart" : "error");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return ResponseVo.failure(start, e.getMessage());
        }
    }

}
