package com.winhc.data.push.service.impl;

import com.aliyuncs.IAcsClient;
import com.aliyuncs.dataworks_public.model.v20180601.CreateManualDagRequest;
import com.aliyuncs.dataworks_public.model.v20180601.CreateManualDagResponse;
import com.aliyuncs.dataworks_public.model.v20180601.SearchManualDagNodeInstanceRequest;
import com.aliyuncs.dataworks_public.model.v20180601.SearchManualDagNodeInstanceResponse;
import com.aliyuncs.http.ProtocolType;
import com.winhc.data.push.bean.DataWorksFlowTask;
import com.winhc.data.push.bean.TaskFlowEnum;
import com.winhc.data.push.bean.TaskParam;
import com.winhc.data.push.configuration.DataWorksAccessProperties;
import com.winhc.data.push.configuration.ProjectParamInit;
import com.winhc.data.push.service.DataWorksService;
import com.winhc.data.push.utils.DateUtils;
import com.winhc.data.push.utils.JsonUtils;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author: XuJiakai
 * @Date: 2020/6/22 11:28
 * @Description:
 */
@Slf4j
@Service
@AllArgsConstructor
public class DataWorksServiceImpl implements DataWorksService {

    private final DataWorksAccessProperties dataWorksAccessProperties;
    private final Map<String, DataWorksFlowTask> dataWorksFlowTaskMap;
    private final IAcsClient client;

    @Override
    public CreateManualDagResponse export2Mongo(String ds) {
        return touch(ProjectParamInit.project, ProjectParamInit.exportMongoFlowName, DateUtils.parseDate(ds), dataWorksFlowTaskMap.get(ProjectParamInit.exportMongoBeanName), new HashMap<>());
    }

    @Override
    public CreateManualDagResponse export2OSS(String ds) {
        return touch(ProjectParamInit.project, ProjectParamInit.exportOssFlowName, DateUtils.parseDate(ds), dataWorksFlowTaskMap.get(ProjectParamInit.exportOssBeanName), new HashMap<>());
    }

    @Override
    public CreateManualDagResponse touch(String project, String flowName, String bizDate, DataWorksFlowTask dataWorksFlowTask, Map<String, String> otherParams) {
        TaskParam build = TaskParam.builder()
                .projectName(project)
                .bizDate(bizDate)
                .flowName(flowName)
                .nodeParam(dataWorksFlowTask.toNodeParam(bizDate, otherParams))
                .build();
        return touch(build);
    }

    @SneakyThrows
    public CreateManualDagResponse triggerWithParam(String projectName
            , String flowName, String bizDate, String dagPara, String nodePara) {
        log.info("触发任务：{}.{} {}", projectName, flowName, bizDate);
        CreateManualDagRequest request = new CreateManualDagRequest();
        request.setProjectName(projectName);
        request.setFlowName(flowName);
        request.setBizdate(bizDate + " 00:00:00");
        request.setRegionId(dataWorksAccessProperties.getRegionId());
        request.setProtocol(ProtocolType.HTTP);
        request.setDagPara(dagPara);
        request.setNodePara(nodePara);
        CreateManualDagResponse response = client.getAcsResponse(request);
        log.info("\n任务结果：\n\trequestId:{},\n\treturnCode:{},\n\treturnErrorSolution:{},\n\treturnMessage:{},\n\treturnValue:{}",
                response.getRequestId()
                , response.getReturnCode()
                , response.getReturnErrorSolution()
                , response.getReturnMessage()
                , response.getReturnValue());
        return response;
    }

    @SneakyThrows
    public CreateManualDagResponse touch(TaskParam taskParam) {
        return triggerWithParam(taskParam.getProjectName(), taskParam.getFlowName(), taskParam.getBizDate(), null, JsonUtils.jsonObjToString(taskParam.getNodeParam()));
    }


    @SneakyThrows
    public Map<String, TaskFlowEnum> query(String projectName, Long dagId) {
        SearchManualDagNodeInstanceRequest searchNodeInstanceListRequest
                = new SearchManualDagNodeInstanceRequest();
        searchNodeInstanceListRequest.setDagId(dagId);
        searchNodeInstanceListRequest.setProjectName(projectName);
        searchNodeInstanceListRequest.setProtocol(ProtocolType.HTTP);
        SearchManualDagNodeInstanceResponse searchResponse = client
                .getAcsResponse(searchNodeInstanceListRequest);
        java.util.List<SearchManualDagNodeInstanceResponse.NodeInsInfo> nodeInsfos = searchResponse.getData();
        for (SearchManualDagNodeInstanceResponse.NodeInsInfo nodeInsInfo : nodeInsfos) {
            TaskFlowEnum code = TaskFlowEnum.getTaskFlowEnumByCode(nodeInsInfo.getStatus());
            log.info("{}:{} {}", nodeInsInfo.getNodeName(), nodeInsInfo.getStatus(), code);
        }
        return nodeInsfos.stream()
                .collect(Collectors.toMap(SearchManualDagNodeInstanceResponse.NodeInsInfo::getNodeName, node -> TaskFlowEnum.getTaskFlowEnumByCode(node.getStatus()), (o1, o2) -> o1));
    }

}
