package com.winhc.data.push.service.impl;

import com.winhc.data.push.bean.*;
import com.winhc.data.push.service.DataWorksService;
import com.winhc.data.push.service.DingTalkService;
import com.winhc.data.push.service.TouchService;
import com.winhc.data.push.utils.DateUtils;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author π
 * @Description:
 * @date 2022/7/21 18:18
 */
@Service
@Slf4j
@AllArgsConstructor
public class TouchServiceImpl implements TouchService {

    private final DataWorksService dataWorksService;
    private final DingTalkService dingTalkService;

    @Override
    public Boolean export2Mongo(String project, String ds) {
        return query(project, dataWorksService.export2Mongo(ds).getReturnValue());
    }

    @Override
    public Boolean export2OSS(String project, String ds) {
        return query(project, dataWorksService.export2OSS(ds).getReturnValue());
    }


    @SneakyThrows
    @Override
    public Boolean query(String projectName, Long dagId) {
        LocalDateTime start = LocalDateTime.now();
        int totalTask = 1;
        int i = 0;
        int successTask = 0;
        int failedTask = 0;
        ArrayList<Long> hours = new ArrayList<>();
        while (true) {
            int awaitTask = 0;
            List<String> empty = new ArrayList<>();
            Map<String, TaskFlowEnum> query = dataWorksService.query(projectName, dagId);
            Map<String, List<String>> status = query.entrySet().stream()
                    .map(e -> {
                        Entry<String, String> entry = new Entry<>();
                        if (e.getValue().equals(TaskFlowEnum.SUCCESS) || e.getValue().equals(TaskFlowEnum.FAILURE)) {
                            entry.setKey(e.getValue().getMsg());
                        } else {
                            entry.setKey("运行中");
                        }
                        entry.setValue(e.getKey());
                        return entry;
                    }).collect(Collectors.groupingBy(Entry::getKey, Collectors.mapping(Entry::getValue, Collectors.toList())));
            List<String> failure = status.getOrDefault(TaskFlowEnum.FAILURE.getMsg(), empty);
            List<String> await = status.getOrDefault(TaskFlowEnum.RUNNING.getMsg(), empty);

            if (failure.size() != 0) {
                log.error("任务失败 node : {}", dagId);
                return false;
            } else {
                if (await.size() != 0) {
                    awaitTask++;
                    LocalDateTime now = LocalDateTime.now();
                    Duration duration = Duration.between(start, now);
                    long hour = duration.toHours();
                    if (hour >= 1 && !hours.contains(hour)) {
                        i++;
                        hours.add(hour);
                        log.info("导出数据阻塞告警 !!!!!!");
                        dingTalkService.sendMessage("导出数据阻塞告警 !!!!!!");
                    }
                } else {
                    return true;
                }
            }

            //超时退出
            if (DateUtils.getCurrentHours() >= 22 || i > 6) {
                log.info("超时程序主动退出 !!!!!!");
                return false;
            }
            log.info("\nawait task:{}\ntotal task:{}\nsuccess task:{}\nfailure task:{}\n"
                    , awaitTask
                    , totalTask
                    , successTask
                    , failedTask
            );
            Thread.sleep(10000);
        }
    }

}
