package com.winhc.data.push.task;


import com.aliyun.openservices.shade.org.apache.commons.lang3.StringUtils;
import com.winhc.data.push.common.TaskEnum;
import com.winhc.data.push.service.DataWorksService;
import com.winhc.data.push.service.SynDataService;
import com.winhc.data.push.utils.BaseUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.bson.Document;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.FileNotFoundException;
import java.util.List;

import static com.winhc.data.push.constant.BaseParam.SYN_ANSHUO_UPLOAD_TASKS;

/**
 * @author π
 * @Description:触发任务
 * @date 2021/6/22 17:07
 */

@Component
@Slf4j
@EnableScheduling
@AllArgsConstructor
public class PushDataTask {


    private final SynDataService synDataService;


    @Scheduled(cron = "*/10 * * * * ?")
    //@Scheduled(cron = "0 /2 * * * ? ")
    public synchronized void start() {
        log.info("start PushDataTask !!! ");
        //获取运行任务
        List<Document> allTask = synDataService.findAllTask(new Document("status", "create"));
        allTask.forEach(task -> {
            String ds = task.getString("ds");
            String tn = task.getString("tn");
            if (StringUtils.isNotBlank(ds) && StringUtils.isNotBlank(tn)) {
                synDataService.push(tn, ds);
            }
        });
        log.info("stop PushDataTask !!! ");

    }

    @Scheduled(cron = "*/10 * * * * ?")
    //@Scheduled(cron = "0 /2 * * * ? ")
    public synchronized void upload() {
        log.info("start upload task !!! ");
        //获取运行任务
        List<Document> allTask = synDataService.findAllTask(SYN_ANSHUO_UPLOAD_TASKS, new Document("status", TaskEnum.STATUS.CREATE.code));
        allTask.forEach(task -> {
            String ds = task.getString("ds");
            if (StringUtils.isNotBlank(ds)) {
                try {
                    synDataService.uploadToOss(ds);
                } catch (FileNotFoundException e) {
                    e.printStackTrace();
                }
            }
        });
        log.info("stop upload task !!! ");

    }
}
