package com.winhc.data.push.utils;

import com.winhc.data.push.bean.DataWorksFlowJob;
import com.winhc.data.push.bean.DataWorksFlowTask;
import com.winhc.data.push.bean.NodeParam;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Component;
import org.yaml.snakeyaml.Yaml;

import java.io.*;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author: XuJiakai
 * @Date: 2020/6/23 14:23
 * @Description:
 */
@SuppressWarnings("all")
@Component
@AllArgsConstructor
public class DataWorksParamUtils {
//    @Autowired
//    ResourceLoader resourceLoader;

    private static final String defaultFileName = "data-works-param.yaml";

    @SneakyThrows
    public List<DataWorksFlowJob> getJobs() {
        InputStream is = null;
        InputStreamReader isr = null;
        BufferedReader br = null;
        try {
//            File file =ResourceUtils.getFile("classpath:data-works-param.yaml");
            Resource resource = new ClassPathResource("data-works-param.yaml");
            is = resource.getInputStream();

//            Resource resource = resourceLoader.getResource("classpath:data-works-param.yaml");
//            is = resource.getInputStream();
            isr = new InputStreamReader(is);
            br = new BufferedReader(isr);
            return parseJobs(br);
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            if (br != null) {
                br.close();
            }
            if (isr != null) {
                isr.close();
            }
            if (is != null) {
                is.close();
            }
        }
    }

    public static List<DataWorksFlowJob> getJobs(String fileName) throws FileNotFoundException {
        return parseJobs(fileName);
    }

    private static List<DataWorksFlowJob> parseJobs(String path) throws FileNotFoundException {
        Reader reader = new FileReader(new File(path));
        return parseJobs(reader);
    }

    private static List<DataWorksFlowJob> parseJobs(Reader reader) throws FileNotFoundException {
        Yaml yml = new Yaml();
        Map map = yml.loadAs(reader, Map.class);

        List j = ((List) map.get("job"));
        List<DataWorksFlowJob> jobs = (List<DataWorksFlowJob>) j
                .stream()

                .map(m -> {
                    String project = ((String) ((Map<String, Object>) m).get("project"));
                    String flow = ((String) ((Map<String, Object>) m).get("flow"));
                    List<DataWorksFlowTask> collect1 = (List<DataWorksFlowTask>) ((List) ((Map<String, Object>) m).get("task"))
                            .stream()
                            .map(t -> {
                                String taskName = ((String) ((Map<String, Object>) t).get("taskName"));
                                List<Map<String, Object>> ll = ((List<Map<String, Object>>) ((Map<String, Object>) t).get("param"));
                                List<NodeParam> nodeParamList = ll.stream()
                                        .map(mm -> {
                                            String nodeId = String.valueOf(mm.remove("_nodeId"));
                                            Map<String, String> collect = mm.entrySet().stream().collect(Collectors.toMap(Map.Entry::getKey, e -> String.valueOf(e.getValue()), (o1, o2) -> o1));
                                            return NodeParam.builder().nodeId(nodeId).param(collect).build();
                                        }).collect(Collectors.toList());
                                return new DataWorksFlowTask(taskName, nodeParamList);
                            })
                            .collect(Collectors.toList());
                    DataWorksFlowJob build = DataWorksFlowJob.builder()
                            .project(project)
                            .flow(flow)
                            .task(collect1)
                            .build();
                    return build;
                })
                .collect(Collectors.toList());

        List<DataWorksFlowJob> collect = jobs.stream().collect(Collectors.groupingBy(m -> m.getFlow() + m.getProject())).values().stream().map(list -> new DataWorksFlowJob(list.get(0).getProject(), list.get(0).getFlow(), list.stream().flatMap(l -> l.getTask().stream()).collect(Collectors.toList()))).collect(Collectors.toList());
        return collect;
    }

    public static void main(String[] args) {
        DataWorksParamUtils bean = new DataWorksParamUtils();
        List<DataWorksFlowJob> jobs = bean.getJobs();
        System.out.println(jobs.toString());
    }
}
